# =================================
# Copyright: CEA-LIST/DIASI/SIALV
# Author : pixano@cea.fr
# License: CECILL-C
# =================================

"""Tasks module."""

from enum import Enum


class Task(Enum):
    """Task base class."""

    pass
