# =================================
# Copyright: CEA-LIST/DIASI/SIALV
# Author : pixano@cea.fr
# License: CECILL-C
# =================================

# ruff: noqa: F401
# ruff: noqa: D104

from .__version__ import __version__
from .settings import PIXANO_INFERENCE_SETTINGS, get_pixano_inference_settings
